@@ -50,6 +50,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <X11/extensions/Xrandr.h>
 #endif
 
+#include <kephal/screens.h>
+
 namespace KWin
 {
 
@@ -478,7 +480,7 @@ bool Workspace::workspaceEvent( XEvent * e )
         case Expose:
             if( compositing()
                 && ( e->xexpose.window == rootWindow()  // root window needs repainting
-                    || overlay != None && e->xexpose.window == overlay )) // overlay needs repainting
+                    || (overlay != None && e->xexpose.window == overlay) )) // overlay needs repainting
                 {
                 addRepaint( e->xexpose.x, e->xexpose.y, e->xexpose.width, e->xexpose.height );
                 }
@@ -634,6 +636,10 @@ bool Client::windowEvent( XEvent* e )
                 i.setOpacity( info->opacity());
                 }
             }
+        if( dirty[ WinInfo::PROTOCOLS2 ] & NET::WM2FrameOverlap )
+            {
+            // ### Inform the decoration
+            }
         }
 
     switch (e->type) 
@@ -1279,7 +1285,13 @@ bool Client::buttonPressEvent( Window w, int button, int state, int x, int y, in
         return true;
         }
     if( w == decorationId())
-        return false; // don't eat decoration events
+        {
+        if( dynamic_cast<KDecorationUnstable*>( decoration ))
+            // New API processes core events FIRST and only passes unused ones to the decoration
+            return processDecorationButtonPress( button, state, x, y, x_root, y_root, true );
+        else
+            return false; // Don't eat old API decoration events
+        }
     if( w == frameId())
         processDecorationButtonPress( button, state, x, y, x_root, y_root );
     return true;
@@ -1288,7 +1300,8 @@ bool Client::buttonPressEvent( Window w, int button, int state, int x, int y, in
 
 // this function processes button press events only after decoration decides not to handle them,
 // unlike buttonPressEvent(), which (when the window is decoration) filters events before decoration gets them
-void Client::processDecorationButtonPress( int button, int /*state*/, int x, int y, int x_root, int y_root )
+bool Client::processDecorationButtonPress( int button, int /*state*/, int x, int y, int x_root, int y_root,
+        bool ignoreMenu )
     {
     Options::MouseCommand com = Options::MouseNothing;
     bool active = isActive();
@@ -1303,7 +1316,8 @@ void Client::processDecorationButtonPress( int button, int /*state*/, int x, int
         com = active ? options->commandActiveTitlebar3() : options->commandInactiveTitlebar3();
     if( button == Button1
         && com != Options::MouseOperationsMenu // actions where it's not possible to get the matching
-        && com != Options::MouseMinimize )  // mouse release event
+        && com != Options::MouseMinimize  // mouse release event
+        && com != Options::MouseClientGroupDrag )
         {
         mode = mousePosition( QPoint( x, y ));
         buttonDown = true;
@@ -1313,7 +1327,19 @@ void Client::processDecorationButtonPress( int button, int /*state*/, int x, int
         startDelayedMoveResize();
         updateCursor();
         }
-    performMouseCommand( com, QPoint( x_root, y_root ));
+    // In the new API the decoration may process the menu action to display an inactive tab's menu.
+    // If the event is unhandled then the core will create one for the active window in the group.
+    if( !ignoreMenu || com != Options::MouseOperationsMenu )
+        performMouseCommand( com, QPoint( x_root, y_root ));
+    return !( // Return events that should be passed to the decoration in the new API
+        com == Options::MouseRaise ||
+        com == Options::MouseOperationsMenu ||
+        com == Options::MouseActivateAndRaise ||
+        com == Options::MouseActivate ||
+        com == Options::MouseActivateRaiseAndPassClick ||
+        com == Options::MouseActivateAndPassClick ||
+        com == Options::MouseClientGroupDrag ||
+        com == Options::MouseNothing );
     }
 
 // called from decoration
@@ -1367,6 +1393,8 @@ bool Client::buttonReleaseEvent( Window w, int /*button*/, int state, int x, int
             QPoint mousepos( x_root - x, y_root - y );
             mode = mousePosition( mousepos );
             }
+        else if( workspace()->decorationSupportsClientGrouping() )
+            return false;
         updateCursor();
         }
     return true;
@@ -1406,6 +1434,39 @@ static bool waitingMotionEvent()
     return was_motion;
     }
 
+// Checks if the mouse cursor is near the edge of the screen and if so activates quick tiling or maximization
+void Client::checkQuickTilingMaximizationZones( int xroot, int yroot )
+    {
+    foreach( Kephal::Screen* screen, Kephal::Screens::self()->screens() )
+        {
+        if( screen->geom().contains( QPoint( xroot, yroot )))
+            {
+            if( options->electricBorderTiling() &&
+                xroot <= screen->geom().x() + 20 )
+                {
+                setElectricBorderMode( ElectricLeftMode );
+                setElectricBorderMaximizing( true );
+                return;
+                }
+            else if( options->electricBorderTiling() &&
+                xroot >= screen->geom().x() + screen->geom().width() - 20 )
+                {
+                setElectricBorderMode( ElectricRightMode );
+                setElectricBorderMaximizing( true );
+                return;
+                }
+            else if( options->electricBorderMaximize() &&
+                yroot <= screen->geom().y() + 5 && isMaximizable() )
+                {
+                setElectricBorderMode( ElectricMaximizeMode );
+                setElectricBorderMaximizing( true );
+                return;
+                }
+            }
+        }
+    setElectricBorderMaximizing( false );
+    }
+
 // return value matters only when filtering events before decoration gets them
 bool Client::motionNotifyEvent( Window w, int /*state*/, int x, int y, int x_root, int y_root )
     {
@@ -1429,8 +1490,12 @@ bool Client::motionNotifyEvent( Window w, int /*state*/, int x, int y, int x_roo
         x = this->x(); // translate from grab window to local coords
         y = this->y();
         }
-    if( !waitingMotionEvent())
+    if( !waitingMotionEvent() )
+        {
         handleMoveResize( x, y, x_root, y_root );
+        if( isMove() && isResizable() )
+            checkQuickTilingMaximizationZones( x_root, y_root );
+        }
     return true;
     }
     
