@@ -20,7 +20,7 @@
 #include "mouse.h"
 
 #include <QLabel>
-#include <QComboBox>
+#include <KComboBox>
 
 #include <QLayout>
 #include <QSizePolicy>
@@ -147,22 +147,21 @@ KTitleBarActionsConfig::KTitleBarActionsConfig (bool _standAlone, KConfig *_conf
   bool leftHandedMouse = ( KGlobalSettings::mouseSettings().handed == KGlobalSettings::KMouseSettings::LeftHanded);
 
   QVBoxLayout *layout = new QVBoxLayout(this);
-  layout->setMargin(0);
-  layout->setSpacing(KDialog::spacingHint());
 
 /** Titlebar doubleclick ************/
 
-  QGridLayout *gLayout = new QGridLayout();
-  layout->addLayout( gLayout );
+  QWidget *titlebarActions = new QWidget(this);
+  QGridLayout *gLayout = new QGridLayout(titlebarActions);
+  layout->addWidget( titlebarActions );
 
-  QComboBox* combo = new QComboBox(this);
-  combo->addItem(i18n("Maximize"));
+  KComboBox* combo = new KComboBox(this);
+  combo->addItem(i18nc("@item:inlistbox behavior on double click", "Maximize"));
   combo->addItem(i18n("Maximize (vertical only)"));
   combo->addItem(i18n("Maximize (horizontal only)"));
-  combo->addItem(i18n("Minimize"));
+  combo->addItem(i18nc("@item:inlistbox behavior on double click", "Minimize"));
   combo->addItem(i18n("Shade"));
   combo->addItem(i18n("Lower"));
-  combo->addItem(i18n("On All Desktops"));
+  combo->addItem(i18nc("@item:inlistbox behavior on double click", "On All Desktops"));
   combo->addItem(i18n("Nothing"));
   combo->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
   connect(combo, SIGNAL(activated(int)), SLOT(changed()));
@@ -178,13 +177,14 @@ KTitleBarActionsConfig::KTitleBarActionsConfig (bool _standAlone, KConfig *_conf
 
 /** Mouse Wheel Events  **************/
   // Titlebar and frame mouse Wheel  
-  QComboBox* comboW = new QComboBox(this);
+  KComboBox* comboW = new KComboBox(this);
   comboW->addItem(i18n("Raise/Lower"));
   comboW->addItem(i18n("Shade/Unshade"));
   comboW->addItem(i18n("Maximize/Restore"));
   comboW->addItem(i18n("Keep Above/Below"));  
   comboW->addItem(i18n("Move to Previous/Next Desktop"));  
   comboW->addItem(i18n("Change Opacity"));  
+  comboW->addItem(i18n("Switch to Window Tab to the Left/Right"));
   comboW->addItem(i18n("Nothing"));  
   comboW->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
   connect(comboW, SIGNAL(activated(int)), SLOT(changed()));
@@ -207,9 +207,6 @@ KTitleBarActionsConfig::KTitleBarActionsConfig (bool _standAlone, KConfig *_conf
                              " titlebar or the frame of a window.") );
 
   grid = new QGridLayout(box);
-  grid->setMargin(KDialog::marginHint());
-  grid->setSpacing(KDialog::spacingHint());
-
 
   strMouseButton1 = i18n("Left button:");
   txtButton1 = i18n("In this row you can customize left click behavior when clicking into"
@@ -249,13 +246,15 @@ KTitleBarActionsConfig::KTitleBarActionsConfig (bool _standAlone, KConfig *_conf
                                " or the frame of an active window.") );
 
   // Titlebar and frame, active, mouse button 1
-  combo = new QComboBox(box);
+  combo = new KComboBox(box);
   combo->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
   grid->addWidget(combo, 1, 1);
   combo->addItem(i18n("Raise"));
   combo->addItem(i18n("Lower"));
   combo->addItem(i18n("Operations Menu"));
   combo->addItem(i18n("Toggle Raise & Lower"));
+  combo->addItem(i18n("Close"));
+  combo->addItem(i18n("Start Window Tab Drag"));
   combo->addItem(i18n("Nothing"));
   connect(combo, SIGNAL(activated(int)), SLOT(changed()));
   coTiAct1 = combo;
@@ -277,10 +276,12 @@ KTitleBarActionsConfig::KTitleBarActionsConfig (bool _standAlone, KConfig *_conf
         << i18n("Lower")
         << i18n("Operations Menu")
         << i18n("Toggle Raise & Lower")
+        << i18n("Close")
+        << i18n("Start Window Tab Drag")
         << i18n("Nothing")
         << i18n("Shade");
 
-  combo = new QComboBox(box);
+  combo = new KComboBox(box);
   combo->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
   grid->addWidget(combo, 2, 1);
   combo->addItems(items);
@@ -289,7 +290,7 @@ KTitleBarActionsConfig::KTitleBarActionsConfig (bool _standAlone, KConfig *_conf
   combo->setWhatsThis( i18n("Behavior on <em>middle</em> click into the titlebar or frame of an <em>active</em> window."));
 
   // Titlebar and frame, active, mouse button 3
-  combo = new QComboBox(box);
+  combo = new KComboBox(box);
   combo->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
   grid->addWidget(combo, 3, 1);
   combo->addItems(items);
@@ -320,9 +321,11 @@ KTitleBarActionsConfig::KTitleBarActionsConfig (bool _standAlone, KConfig *_conf
          << i18n("Operations Menu")
          << i18n("Raise")
          << i18n("Lower")
+         << i18n("Close")
+         << i18n("Start Window Tab Drag")
          << i18n("Nothing");
 
-  combo = new QComboBox(box);
+  combo = new KComboBox(box);
   combo->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
   grid->addWidget(combo, 1, 2);
   combo->addItems(items);
@@ -330,7 +333,7 @@ KTitleBarActionsConfig::KTitleBarActionsConfig (bool _standAlone, KConfig *_conf
   coTiInAct1 = combo;
   combo->setWhatsThis( txtButton1);
 
-  combo = new QComboBox(box);
+  combo = new KComboBox(box);
   combo->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
   grid->addWidget(combo, 2, 2);
   combo->addItems(items);
@@ -338,7 +341,7 @@ KTitleBarActionsConfig::KTitleBarActionsConfig (bool _standAlone, KConfig *_conf
   coTiInAct2 = combo;
   combo->setWhatsThis( i18n("Behavior on <em>middle</em> click into the titlebar or frame of an <em>inactive</em> window."));
 
-  combo = new QComboBox(box);
+  combo = new KComboBox(box);
   combo->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
   grid->addWidget(combo, 3, 2);
   combo->addItems(items);
@@ -355,8 +358,6 @@ KTitleBarActionsConfig::KTitleBarActionsConfig (bool _standAlone, KConfig *_conf
     i18n("Here you can customize behavior when clicking on the maximize button.") );
 
   QHBoxLayout* hlayout = new QHBoxLayout(box);
-  hlayout->setMargin(KDialog::marginHint());
-  hlayout->setSpacing(KDialog::spacingHint());
 
   QString strMouseButton[] = {
     i18n("Left button:"),
@@ -425,6 +426,8 @@ const char* const tbl_TiAc[] = {
     "Lower",
     "Operations menu",
     "Toggle raise and lower",
+    "Close",
+    "Start window tab drag",
     "Nothing",
     "Shade",
     "" };
@@ -437,6 +440,8 @@ const char* const tbl_TiInAc[] = {
     "Operations menu",
     "Raise",
     "Lower",
+    "Close",
+    "Start window tab drag",
     "Nothing",
     "" };
 
@@ -476,6 +481,7 @@ const char* tbl_TiWAc[] = {
     "Above/Below",
     "Previous/Next Desktop",
     "Change Opacity",
+    "Switch to Window Tab to the Left/Right",
     "Nothing",
     "" };
 
@@ -486,6 +492,7 @@ const char* tbl_AllW[] = {
     "Above/Below",
     "Previous/Next Desktop",
     "Change Opacity",
+    "Switch to Window Tab to the Left/Right",
     "Nothing",
     "" };
 
@@ -516,7 +523,7 @@ static int tbl_txt_lookup( const char* const arr[], const char* txt )
     return 0;
 }
 
-void KTitleBarActionsConfig::setComboText( QComboBox* combo, const char*txt )
+void KTitleBarActionsConfig::setComboText( KComboBox* combo, const char*txt )
 {
     if( combo == coTiDbl )
         combo->setCurrentIndex( tbl_txt_lookup( tbl_TiDbl, txt ));
@@ -560,6 +567,16 @@ const char* KTitleBarActionsConfig::functionMax( int i )
     return tbl_num_lookup( tbl_Max, i );
 }
 
+void KTitleBarActionsConfig::showEvent( QShowEvent *ev )
+{
+    if ( !standAlone ) {
+        // Workaround KCModule::showEvent() calling load(), see bug 163817
+        QWidget::showEvent( ev );
+        return;
+    }
+    KCModule::showEvent( ev );
+}
+
 void KTitleBarActionsConfig::load()
 {
   KConfigGroup windowsConfig(config, "Windows");
@@ -569,11 +586,11 @@ void KTitleBarActionsConfig::load()
 
   KConfigGroup cg(config, "MouseBindings");
   setComboText(coTiAct1,cg.readEntry("CommandActiveTitlebar1","Raise").toAscii());
-  setComboText(coTiAct2,cg.readEntry("CommandActiveTitlebar2","Lower").toAscii());
+  setComboText(coTiAct2,cg.readEntry("CommandActiveTitlebar2","Start Window Tab Drag").toAscii());
   setComboText(coTiAct3,cg.readEntry("CommandActiveTitlebar3","Operations menu").toAscii());
-  setComboText(coTiAct4,cg.readEntry("CommandTitlebarWheel","Nothing").toAscii());  
+  setComboText(coTiAct4,cg.readEntry("CommandTitlebarWheel","Switch to Window Tab to the Left/Right").toAscii());
   setComboText(coTiInAct1,cg.readEntry("CommandInactiveTitlebar1","Activate and raise").toAscii());
-  setComboText(coTiInAct2,cg.readEntry("CommandInactiveTitlebar2","Activate and lower").toAscii());
+  setComboText(coTiInAct2,cg.readEntry("CommandInactiveTitlebar2","Start Window Tab Drag").toAscii());
   setComboText(coTiInAct3,cg.readEntry("CommandInactiveTitlebar3","Operations menu").toAscii());
 }
 
@@ -608,11 +625,11 @@ void KTitleBarActionsConfig::defaults()
 {
   setComboText(coTiDbl, "Shade");
   setComboText(coTiAct1,"Raise");
-  setComboText(coTiAct2,"Lower");
+  setComboText(coTiAct2,"Start Window Tab Drag");
   setComboText(coTiAct3,"Operations menu");
-  setComboText(coTiAct4,"Nothing");    
+  setComboText(coTiAct4,"Switch to Window Tab to the Left/Right");
   setComboText(coTiInAct1,"Activate and raise");
-  setComboText(coTiInAct2,"Activate and lower");
+  setComboText(coTiInAct2,"Start Window Tab Drag");
   setComboText(coTiInAct3,"Operations menu");
   for (int t = 0; t < 3; ++t)
     setComboText(coMax[t], tbl_Max[t]);
@@ -630,8 +647,6 @@ KWindowActionsConfig::KWindowActionsConfig (bool _standAlone, KConfig *_config,
   bool leftHandedMouse = ( KGlobalSettings::mouseSettings().handed == KGlobalSettings::KMouseSettings::LeftHanded);
 
   QVBoxLayout *layout = new QVBoxLayout(this);
-  layout->setMargin(0);
-  layout->setSpacing(KDialog::spacingHint());
 
 /**  Inactive inner window ******************/
 
@@ -642,8 +657,6 @@ KWindowActionsConfig::KWindowActionsConfig (bool _standAlone, KConfig *_config,
                              " inner window ('inner' means: not titlebar, not frame).") );
 
   QGridLayout *gridLayout = new QGridLayout(box);
-  gridLayout->setMargin(KDialog::marginHint());
-  gridLayout->setSpacing(KDialog::spacingHint());
 
   strMouseButton1 = i18n("Left button:");
   txtButton1 = i18n("In this row you can customize left click behavior when clicking into"
@@ -686,7 +699,7 @@ KWindowActionsConfig::KWindowActionsConfig (bool _standAlone, KConfig *_config,
           << i18n("Activate")
           << i18n("Activate & Raise");
 
-  QComboBox* combo = new QComboBox(box);
+  KComboBox* combo = new KComboBox(box);
   coWin1 = combo;
   combo->addItems(items);
   connect(combo, SIGNAL(activated(int)), SLOT(changed()));
@@ -698,7 +711,7 @@ KWindowActionsConfig::KWindowActionsConfig (bool _standAlone, KConfig *_config,
   gridLayout->addWidget(label, 0, 0);
   gridLayout->addWidget(combo, 0, 1);
 
-  combo = new QComboBox(box);
+  combo = new KComboBox(box);
   combo->addItems(items);
   connect(combo, SIGNAL(activated(int)), SLOT(changed()));
   coWin2 = combo;
@@ -710,7 +723,7 @@ KWindowActionsConfig::KWindowActionsConfig (bool _standAlone, KConfig *_config,
   gridLayout->addWidget(label, 1, 0);
   gridLayout->addWidget(combo, 1, 1);
 
-  combo = new QComboBox(box);
+  combo = new KComboBox(box);
   combo->addItems(items);
   connect(combo, SIGNAL(activated(int)), SLOT(changed()));
   coWin3 = combo;
@@ -727,7 +740,7 @@ KWindowActionsConfig::KWindowActionsConfig (bool _standAlone, KConfig *_config,
           << i18n("Activate & Scroll")
           << i18n("Activate, Raise & Scroll");
   
-  combo = new QComboBox(box);
+  combo = new KComboBox(box);
   combo->addItems(items);
   connect(combo, SIGNAL(activated(int)), SLOT(changed()));
   coWinWheel = combo;
@@ -751,14 +764,8 @@ KWindowActionsConfig::KWindowActionsConfig (bool _standAlone, KConfig *_config,
                              " a window while pressing a modifier key."));
 
   QHBoxLayout* innerLay=new QHBoxLayout(box);
-  innerLay->setContentsMargins(0,0,0,0);
-  innerLay->setSpacing(KDialog::spacingHint());
   QHBoxLayout* fLay=new QHBoxLayout;
-  fLay->setMargin(KDialog::marginHint());
-  fLay->setSpacing(KDialog::spacingHint());
   gridLayout = new QGridLayout;
-  gridLayout->setMargin(KDialog::marginHint());
-  gridLayout->setSpacing(KDialog::spacingHint());
   innerLay->addLayout(fLay);
   innerLay->addLayout(gridLayout);
 
@@ -782,7 +789,7 @@ KWindowActionsConfig::KWindowActionsConfig (bool _standAlone, KConfig *_config,
   }
 
   // Combo's
-  combo = new QComboBox(box);
+  combo = new KComboBox(box);
   combo->addItem(i18n("Meta"));
   combo->addItem(i18n("Alt"));
   connect(combo, SIGNAL(activated(int)), SLOT(changed()));
@@ -808,7 +815,7 @@ KWindowActionsConfig::KWindowActionsConfig (bool _standAlone, KConfig *_config,
         << i18n("Minimize")
         << i18n("Nothing");
 
-  combo = new QComboBox(box);
+  combo = new KComboBox(box);
   combo->addItems(items);
   connect(combo, SIGNAL(activated(int)), SLOT(changed()));
   coAll1 = combo;
@@ -821,7 +828,7 @@ KWindowActionsConfig::KWindowActionsConfig (bool _standAlone, KConfig *_config,
   gridLayout->addWidget(combo, 0, 1);
 
 
-  combo = new QComboBox(box);
+  combo = new KComboBox(box);
   combo->addItems(items);
   connect(combo, SIGNAL(activated(int)), SLOT(changed()));
   coAll2 = combo;
@@ -833,7 +840,7 @@ KWindowActionsConfig::KWindowActionsConfig (bool _standAlone, KConfig *_config,
   gridLayout->addWidget(label, 1, 0);
   gridLayout->addWidget(combo, 1, 1);
 
-  combo = new QComboBox(box);
+  combo = new KComboBox(box);
   combo->addItems(items);
   connect(combo, SIGNAL(activated(int)), SLOT(changed()));
   coAll3 =  combo;
@@ -846,13 +853,14 @@ KWindowActionsConfig::KWindowActionsConfig (bool _standAlone, KConfig *_config,
   gridLayout->addWidget(combo, 2, 1);
 
 
-  combo = new QComboBox(box);
+  combo = new KComboBox(box);
   combo->addItem(i18n("Raise/Lower"));
   combo->addItem(i18n("Shade/Unshade"));
   combo->addItem(i18n("Maximize/Restore"));
   combo->addItem(i18n("Keep Above/Below"));  
   combo->addItem(i18n("Move to Previous/Next Desktop"));  
   combo->addItem(i18n("Change Opacity"));  
+  combo->addItem(i18n("Switch to Window Tab to the Left/Right"));
   combo->addItem(i18n("Nothing"));  
   connect(combo, SIGNAL(activated(int)), SLOT(changed()));
   coAllW =  combo;
@@ -876,7 +884,7 @@ KWindowActionsConfig::~KWindowActionsConfig()
     delete config;
 }
 
-void KWindowActionsConfig::setComboText( QComboBox* combo, const char*txt )
+void KWindowActionsConfig::setComboText( KComboBox* combo, const char*txt )
 {
     if( combo == coWin1 || combo == coWin2 || combo == coWin3 )
         combo->setCurrentIndex( tbl_txt_lookup( tbl_Win, txt ));
@@ -917,6 +925,15 @@ const char* KWindowActionsConfig::functionAllW(int i)
     return tbl_num_lookup( tbl_AllW, i );
 }
 
+void KWindowActionsConfig::showEvent( QShowEvent *ev )
+{
+    if ( !standAlone ) {
+        QWidget::showEvent( ev );
+        return;
+    }
+    KCModule::showEvent( ev );
+}
+
 void KWindowActionsConfig::load()
 {
   KConfigGroup cg(config, "MouseBindings");
