@@ -87,6 +87,8 @@ unsigned long Options::updateSettings()
 
     rollOverDesktops = config.readEntry("RollOverDesktops", true);
 
+    legacyFullscreenSupport = config.readEntry( "LegacyFullscreenSupport", false );
+
 //    focusStealingPreventionLevel = config.readEntry( "FocusStealingPreventionLevel", 2 );
     // TODO use low level for now
     focusStealingPreventionLevel = config.readEntry( "FocusStealingPreventionLevel", 1 );
@@ -143,6 +145,9 @@ unsigned long Options::updateSettings()
     electric_borders = config.readEntry("ElectricBorders", 0);
     electric_border_delay = config.readEntry("ElectricBorderDelay", 150);
     electric_border_cooldown = config.readEntry("ElectricBorderCooldown", 350);
+    electric_border_pushback_pixels = config.readEntry("ElectricBorderPushbackPixels", 1);
+    electric_border_maximize = config.readEntry("ElectricBorderMaximize", true);
+    electric_border_tiling = config.readEntry("ElectricBorderTiling", true);
 
     OpTitlebarDblClick = windowOperation( config.readEntry("TitlebarDoubleClickCommand", "Maximize"), true );
     setOpMaxButtonLeftClick( windowOperation( config.readEntry("MaximizeButtonLeftClickCommand", "Maximize"), true ));
@@ -164,17 +169,22 @@ unsigned long Options::updateSettings()
 
     killPingTimeout = config.readEntry( "KillPingTimeout", 5000 );
     hideUtilityWindowsForInactive = config.readEntry( "HideUtilityWindowsForInactive", true);
+    inactiveTabsSkipTaskbar = config.readEntry( "InactiveTabsSkipTaskbar", false );
+    autogroupSimilarWindows = config.readEntry( "AutogroupSimilarWindows", false );
+    autogroupInForeground = config.readEntry( "AutogroupInForeground", true );
     showDesktopIsMinimizeAll = config.readEntry( "ShowDesktopIsMinimizeAll", false );
 
+    borderless_maximized_windows = config.readEntry( "BorderlessMaximizedWindows", false );
+
     // Mouse bindings
     config = KConfigGroup( _config, "MouseBindings" );
     CmdActiveTitlebar1 = mouseCommand(config.readEntry("CommandActiveTitlebar1","Raise"), true );
-    CmdActiveTitlebar2 = mouseCommand(config.readEntry("CommandActiveTitlebar2","Lower"), true );
+    CmdActiveTitlebar2 = mouseCommand(config.readEntry("CommandActiveTitlebar2","Start Window Tab Drag"), true );
     CmdActiveTitlebar3 = mouseCommand(config.readEntry("CommandActiveTitlebar3","Operations menu"), true );
     CmdInactiveTitlebar1 = mouseCommand(config.readEntry("CommandInactiveTitlebar1","Activate and raise"), true );
-    CmdInactiveTitlebar2 = mouseCommand(config.readEntry("CommandInactiveTitlebar2","Activate and lower"), true );
+    CmdInactiveTitlebar2 = mouseCommand(config.readEntry("CommandInactiveTitlebar2","Start Window Tab Drag"), true );
     CmdInactiveTitlebar3 = mouseCommand(config.readEntry("CommandInactiveTitlebar3","Operations menu"), true );
-    CmdTitlebarWheel = mouseWheelCommand(config.readEntry("CommandTitlebarWheel","Nothing"));
+    CmdTitlebarWheel = mouseWheelCommand(config.readEntry("CommandTitlebarWheel","Switch to Window Tab to the Left/Right"));
     CmdWindow1 = mouseCommand(config.readEntry("CommandWindow1","Activate, raise and pass click"), false );
     CmdWindow2 = mouseCommand(config.readEntry("CommandWindow2","Activate and pass click"), false );
     CmdWindow3 = mouseCommand(config.readEntry("CommandWindow3","Activate and pass click"), false );
@@ -207,20 +217,27 @@ unsigned long Options::updateSettings()
 // KDE4 this probably needs to be done manually in clients
 
     // Driver-specific config detection
-    CompositingPrefs prefs;
-    prefs.detect();
-    reloadCompositingSettings( prefs );
+    reloadCompositingSettings();
 
     return changed;
     }
 
-void Options::reloadCompositingSettings(const CompositingPrefs& prefs)
+void Options::reloadCompositingSettings()
     {
     KSharedConfig::Ptr _config = KGlobal::config();
     KConfigGroup config(_config, "Compositing");
 
+    // do not even detect compositing preferences if explicitly disabled
+    if( config.hasKey( "Enabled" ) && !config.readEntry( "Enabled", true ))
+        {
+        useCompositing = false;
+        return;
+        }
     // Compositing settings
-    useCompositing = config.readEntry("Enabled", prefs.enableCompositing());
+    CompositingPrefs prefs;
+    prefs.detect();
+    useCompositing = config.readEntry( "Enabled" , prefs.recommendCompositing());
+
     QString compositingBackend = config.readEntry("Backend", "OpenGL");
     if( compositingBackend == "XRender" )
         compositingMode = XRenderCompositing;
@@ -263,6 +280,7 @@ ElectricBorderAction Options::electricBorderAction( const QString& name )
     {
     QString lowerName = name.toLower();
     if( lowerName == "dashboard" ) return ElectricActionDashboard;
+    else if( lowerName == "showdesktop" ) return ElectricActionShowDesktop;
     return ElectricActionNone;
     }
 
@@ -318,6 +336,8 @@ Options::MouseCommand Options::mouseCommand(const QString &name, bool restricted
     if (lowerName == "resize") return restricted ? MouseResize : MouseUnrestrictedResize;
     if (lowerName == "shade") return MouseShade;
     if (lowerName == "minimize") return MouseMinimize;
+    if (lowerName == "start window tab drag") return MouseClientGroupDrag;
+    if (lowerName == "close") return MouseClose;
     if (lowerName == "nothing") return MouseNothing;
     return MouseNothing;
     }
@@ -331,7 +351,9 @@ Options::MouseWheelCommand Options::mouseWheelCommand(const QString &name)
     if (lowerName == "above/below") return MouseWheelAboveBelow;
     if (lowerName == "previous/next desktop") return MouseWheelPreviousNextDesktop;
     if (lowerName == "change opacity") return MouseWheelChangeOpacity;
-    return MouseWheelNothing;
+    if (lowerName == "switch to window tab to the left/right") return MouseWheelChangeGroupWindow;
+    if (lowerName == "nothing") return MouseWheelNothing;
+    return MouseWheelChangeGroupWindow;
     }
 
 bool Options::showGeometryTip()
@@ -359,6 +381,9 @@ ElectricBorderAction Options::electricBorderAction( ElectricBorder edge )
             return electric_border_left;
         case ElectricTopLeft:
             return electric_border_top_left;
+        default:
+            // fallthrough
+            break;
         }
     return ElectricActionNone;
     }
@@ -399,6 +424,8 @@ Options::MouseCommand Options::wheelToMouseCommand( MouseWheelCommand com, int d
             return delta > 0 ? MousePreviousDesktop : MouseNextDesktop;
         case MouseWheelChangeOpacity:
             return delta > 0 ? MouseOpacityMore : MouseOpacityLess;
+        case MouseWheelChangeGroupWindow:
+            return delta > 0 ? MouseLeftGroupWindow : MouseRightGroupWindow;
         default:
             return MouseNothing;
         }
