@@ -109,11 +109,11 @@ QMenu* Workspace::clientPopup()
         action->setData( Options::SetupWindowShortcutOp );
 
         action = advanced_popup->addAction( i18n("&Special Window Settings...") );
-        action->setIcon( KIcon( "wizard" ) );
+        action->setIcon( KIcon( "preferences-system-windows-actions" ) );
         action->setData( Options::WindowRulesOp );
 
         action = advanced_popup->addAction( i18n("&Special Application Settings...") );
-        action->setIcon( KIcon( "wizard" ) );
+        action->setIcon( KIcon( "preferences-system-windows-actions" ) );
         action->setData( Options::ApplicationRulesOp );
 
         trans_popup = 0;
@@ -167,6 +167,27 @@ QMenu* Workspace::clientPopup()
         mShadeOpAction->setCheckable( true );
         mShadeOpAction->setData( Options::ShadeOp );
 
+        popup->addSeparator();
+
+        // Actions for window tabbing
+        if( decorationSupportsClientGrouping() )
+            {
+            mRemoveTabGroup = popup->addAction( i18n("Remove &from group") );
+            kaction = qobject_cast<KAction*>( keys->action("Remove TabGroup") );
+            if( kaction!=0 )
+                mRemoveTabGroup->setShortcut( kaction->globalShortcut().primary() );
+            mRemoveTabGroup->setData( Options::RemoveClientFromGroupOp );
+
+            mCloseGroup = popup->addAction( i18n("Close entire &group") );
+            mCloseGroup->setIcon( KIcon( "window-close" ) );
+            kaction = qobject_cast<KAction*>( keys->action("Close TabGroup") );
+            if( kaction!=0 )
+                mCloseGroup->setShortcut( kaction->globalShortcut().primary() );
+            mCloseGroup->setData( Options::CloseClientGroupOp );
+
+            popup->addSeparator();
+            }
+
         action = popup->addMenu( advanced_popup );
         action->setText( i18n("Ad&vanced") );
 
@@ -196,6 +217,8 @@ void Workspace::discardPopup()
     delete popup;
     popup = NULL; 
     desk_popup = NULL;
+    switch_to_tab_popup = NULL;
+    add_tabs_popup = NULL;
     }
 
 void Workspace::setPopupClientOpacity( QAction* action )
@@ -240,6 +263,23 @@ void Workspace::clientPopupAboutToShow()
     mNoBorderOpAction->setChecked( active_popup_client->noBorder() );
     mMinimizeOpAction->setEnabled( active_popup_client->isMinimizable() );
     mCloseOpAction->setEnabled( active_popup_client->isCloseable() );
+
+    delete switch_to_tab_popup;
+    switch_to_tab_popup = 0;
+    delete add_tabs_popup;
+    add_tabs_popup = 0;
+    if( decorationSupportsClientGrouping() )
+        {
+        const int tabGroupSize = active_popup_client->clientGroup() ?
+                                 active_popup_client->clientGroup()->items().count() : 1;
+        if( tabGroupSize > 1 )
+            initSwitchToTab();
+        initAddToTabGroup();
+
+        mRemoveTabGroup->setVisible( tabGroupSize > 1 );
+        mCloseGroup->setVisible( tabGroupSize > 1 );
+        }
+
     if( trans_popup != NULL )
         {
         foreach( QAction* action, trans_popup->actions())
@@ -252,6 +292,118 @@ void Workspace::clientPopupAboutToShow()
         }
     }
 
+void Workspace::initSwitchToTab()
+    {
+    if( switch_to_tab_popup )
+        return;
+    switch_to_tab_popup = new QMenu( popup );
+    switch_to_tab_popup->setFont( KGlobalSettings::menuFont() );
+    connect( switch_to_tab_popup, SIGNAL( triggered( QAction* ) ),
+             this, SLOT( slotSwitchToTab( QAction* ) ) );
+    connect( switch_to_tab_popup, SIGNAL( aboutToShow() ),
+             this, SLOT( switchToTabPopupAboutToShow() ) );
+
+    QAction* action = switch_to_tab_popup->menuAction();
+    popup->insertAction( mRemoveTabGroup, action );
+    action->setText( i18n("Switch to window tab") );
+    }
+
+void Workspace::slotSwitchToTab( QAction* action )
+    {
+    int side = action->data().toInt();
+    int c_id = active_popup_client->clientGroup()->indexOfVisibleClient();
+    int size = active_popup_client->clientGroup()->clients().count();
+    if( side == 0 ) // Left
+        {
+        if( c_id > 0 )
+            active_popup_client->clientGroup()->setVisible( c_id - 1 );
+        else
+            active_popup_client->clientGroup()->setVisible( size - 1 );
+        }
+    else if( side == 1 ) // Right
+        {
+        if( c_id < size - 1 )
+            active_popup_client->clientGroup()->setVisible( c_id + 1 );
+        else
+            active_popup_client->clientGroup()->setVisible( 0 );
+        }
+    else // Find the client
+        {
+        side -= 2;
+        for( QList<ClientGroup*>::const_iterator i = clientGroups.constBegin(); i != clientGroups.constEnd(); ++i )
+            {
+            if( (*i)->contains( active_popup_client ))
+                {
+                (*i)->setVisible( side );
+                break;
+                }
+            }
+        }
+    }
+
+void Workspace::switchToTabPopupAboutToShow()
+    {
+    if( !switch_to_tab_popup )
+        return;
+    switch_to_tab_popup->clear();
+    QAction* action = switch_to_tab_popup->addAction( i18n( "To the left" ));
+    action->setData( 0 );
+    action = switch_to_tab_popup->addAction( i18n( "To the right" ));
+    action->setData( 1 );
+    switch_to_tab_popup->addSeparator();
+    int index = 2;
+    foreach( Client* c, active_popup_client->clientGroup()->clients() )
+        {
+        if( c != active_popup_client->clientGroup()->visible() )
+            {
+            action = switch_to_tab_popup->addAction( c->caption() );
+            action->setData( index );
+            }
+        index++;
+        }
+
+    }
+
+void Workspace::initAddToTabGroup()
+    {
+    if( add_tabs_popup )
+        return;
+    add_tabs_popup = new QMenu( popup );
+    add_tabs_popup->setFont( KGlobalSettings::menuFont() );
+    connect( add_tabs_popup, SIGNAL( triggered( QAction* ) ),
+             this, SLOT( slotAddToTabGroup( QAction* ) ) ); // Merge to a group
+    connect( add_tabs_popup, SIGNAL( aboutToShow() ),
+             this, SLOT( groupTabPopupAboutToShow() ) ); // Show the possible groups to add
+
+    QAction* action = add_tabs_popup->menuAction();
+    popup->insertAction( mRemoveTabGroup, action );
+    action->setText( i18n("Move window to group") );
+    }
+
+void Workspace::slotAddToTabGroup( QAction* action )
+    {
+    if( !action->data().isValid() )
+        return;
+    moveItemToClientGroup( active_popup_client->clientGroup(),
+        active_popup_client->clientGroup()->indexOfClient( active_popup_client ),
+        clientGroups[action->data().toInt()], -1 );
+    }
+
+void Workspace::groupTabPopupAboutToShow()
+    {
+    if( !add_tabs_popup )
+        return;
+    add_tabs_popup->clear();
+    int index = 0;
+    for( QList<ClientGroup*>::const_iterator i = clientGroups.constBegin(); i != clientGroups.constEnd(); i++, index++ )
+        {
+        if( !(*i)->contains( active_popup_client ))
+            {
+            QAction* action = add_tabs_popup->addAction( (*i)->visible()->caption() );
+            action->setData( index );
+            }
+        }
+    }
 
 void Workspace::initDesktopPopup()
     {
@@ -327,9 +479,7 @@ void Workspace::initShortcuts()
     // a separate KActionCollection is needed for the shortcut for disabling global shortcuts,
     // otherwise it would also disable itself
     disable_shortcuts_keys = new KActionCollection( this );
-#ifdef __GNUC__
-#warning TODO PORT ME (KGlobalAccel related)
-#endif
+    // TODO: PORT ME (KGlobalAccel related)
     // FIXME KAccel port... needed?
     //disable_shortcuts_keys->disableBlocking( true );
 #define IN_KWIN
@@ -339,9 +489,7 @@ void Workspace::initShortcuts()
 
 void Workspace::readShortcuts()
     {
-#ifdef __GNUC__
-#warning TODO PORT ME (KGlobalAccel related)
-#endif
+    // TODO: PORT ME (KGlobalAccel related)
     //KGlobalAccel::self()->readSettings();
 
     KAction *kaction = qobject_cast<KAction*>( keys->action("Walk Through Desktops") );
@@ -385,6 +533,36 @@ void Workspace::readShortcuts()
         cutWalkThroughWindowsReverse = kaction->globalShortcut();
         connect( kaction, SIGNAL(globalShortcutChanged(QKeySequence)), this, SLOT(slotWalkBackThroughWindowsKeyChanged(QKeySequence)));
         }
+
+    kaction = qobject_cast<KAction*>( keys->action("Walk Through Window Tabs") );
+    if( kaction != 0 )
+        {
+        cutWalkThroughGroupWindows = kaction->globalShortcut();
+        connect( kaction, SIGNAL( globalShortcutChanged( QKeySequence ) ), this,
+        SLOT( slotMoveToTabRightKeyChanged( QKeySequence ) ) );
+        }
+
+    kaction = qobject_cast<KAction*>( keys->action("Walk Through Window Tabs (Reverse)") );
+    if( kaction != 0 )
+        {
+        cutWalkThroughGroupWindowsReverse = kaction->globalShortcut();
+        connect( kaction, SIGNAL( globalShortcutChanged( QKeySequence ) ), this,
+        SLOT( slotMoveToTabLeftKeyChanged( QKeySequence ) ) );
+        }
+
+    kaction = qobject_cast<KAction*>( keys->action("Walk Through Windows Alternative") );
+    if ( kaction!=0 )
+        {
+        cutWalkThroughWindowsAlternative = kaction->globalShortcut();
+        connect( kaction, SIGNAL(globalShortcutChanged(QKeySequence)), this, SLOT(slotWalkThroughWindowsAlternativeKeyChanged(QKeySequence)));
+        }
+
+    kaction = qobject_cast<KAction*>( keys->action("Walk Through Windows Alternative (Reverse)") );
+    if ( kaction!=0 )
+        {
+        cutWalkThroughWindowsAlternativeReverse = kaction->globalShortcut();
+        connect( kaction, SIGNAL(globalShortcutChanged(QKeySequence)), this, SLOT(slotWalkBackThroughWindowsAlternativeKeyChanged(QKeySequence)));
+        }
     discardPopup(); // so that it's recreated next time
     }
 
@@ -392,12 +570,10 @@ void Workspace::readShortcuts()
 void Workspace::setupWindowShortcut( Client* c )
     {
     assert( client_keys_dialog == NULL );
-#ifdef __GNUC__
-#warning TODO PORT ME (KGlobalAccel related)
-#endif
-//    keys->setEnabled( false );
-//    disable_shortcuts_keys->setEnabled( false );
-//    client_keys->setEnabled( false );
+    // TODO: PORT ME (KGlobalAccel related)
+    //keys->setEnabled( false );
+    //disable_shortcuts_keys->setEnabled( false );
+    //client_keys->setEnabled( false );
     client_keys_dialog = new ShortcutDialog( c->shortcut().primary());
     client_keys_client = c;
     connect( client_keys_dialog, SIGNAL( dialogDone( bool )), SLOT( setupWindowShortcutDone( bool )));
@@ -572,11 +748,63 @@ void Workspace::performWindowOperation( Client* c, Options::WindowOperation op )
         case Options::LowerOp:
             lowerClient(c);
             break;
+        case Options::ClientGroupDragOp: // Handled by decoration itself
         case Options::NoOp:
             break;
+        case Options::RemoveClientFromGroupOp:
+            c->clientGroup()->remove( c );
+            break;
+        case Options::MoveClientInGroupLeftOp:
+            {
+            int c_id = c->clientGroup()->indexOfClient( c );
+            int size = c->clientGroup()->clients().count();
+            if( c_id > 0 )
+                c->clientGroup()->setVisible( c_id - 1 );
+            else
+                c->clientGroup()->setVisible( size - 1 );
+            break;
+            }
+        case Options::MoveClientInGroupRightOp:
+            {
+            int c_id = c->clientGroup()->indexOfClient( c );
+            int size = c->clientGroup()->clients().count();
+            if( c_id < size - 1 )
+                c->clientGroup()->setVisible( c_id + 1 );
+            else
+                c->clientGroup()->setVisible( 0 );
+            break;
+            }
+        case Options::CloseClientGroupOp:
+            c->clientGroup()->closeAll();
         }
     }
 
+/**
+ * Called by the decoration in the new API to determine what buttons the user has configured for
+ * window tab dragging and the operations menu.
+ */
+Options::WindowOperation Client::mouseButtonToWindowOperation( Qt::MouseButtons button )
+    {
+    Options::MouseCommand com = Options::MouseNothing;
+    bool active = isActive();
+    if( !wantsInput() ) // we cannot be active, use it anyway
+        active = true;
+
+    if( button == Qt::LeftButton )
+        com = active ? options->commandActiveTitlebar1() : options->commandInactiveTitlebar1();
+    else if( button == Qt::MidButton )
+        com = active ? options->commandActiveTitlebar2() : options->commandInactiveTitlebar2();
+    else if( button == Qt::RightButton )
+        com = active ? options->commandActiveTitlebar3() : options->commandInactiveTitlebar3();
+
+    // TODO: Complete the list
+    if( com == Options::MouseClientGroupDrag )
+        return Options::ClientGroupDragOp;
+    if( com == Options::MouseOperationsMenu )
+        return Options::OperationsOp;
+    return Options::NoOp;
+    }
+
 /*!
   Performs a mouse command on this client (see options.h)
  */
@@ -734,6 +962,30 @@ bool Client::performMouseCommand( Options::MouseCommand command, const QPoint &g
             if( !isDesktop() ) // No point in changing the opacity of the desktop
                 setOpacity( qMax( opacity() - 0.1, 0.1 ));
             break;
+        case Options::MouseLeftGroupWindow:
+            {
+            int c_id = clientGroup()->indexOfClient( this );
+            int size = clientGroup()->clients().count();
+            if( c_id > 0 )
+                clientGroup()->setVisible( c_id - 1 );
+            else
+                clientGroup()->setVisible( size - 1 );
+            }
+            break;
+        case Options::MouseRightGroupWindow:
+            {
+            int c_id = clientGroup()->indexOfClient( this );
+            int size = clientGroup()->clients().count();
+            if( c_id < size - 1 )
+                clientGroup()->setVisible( c_id + 1 );
+            else
+                clientGroup()->setVisible( 0 );
+            }
+            break;
+        case Options::MouseClose:
+            closeWindow();
+            break;
+        case Options::MouseClientGroupDrag:
         case Options::MouseNothing:
             replay = true;
             break;
@@ -1031,6 +1283,14 @@ void Workspace::slotSetupWindowShortcut()
         performWindowOperation( c, Options::SetupWindowShortcutOp );
     }
 
+/*!
+  Toggles show desktop
+ */
+void Workspace::slotToggleShowDesktop()
+    {
+    setShowingDesktop( !showingDesktop() );
+    }
+
 /*!
   Move window to next desktop
  */
@@ -1135,6 +1395,36 @@ void Workspace::slotWindowToDesktopDown()
         }
     }
 
+void Workspace::slotSwitchToTabRight()
+    {
+    if( !active_client || !active_client->clientGroup() )
+        return;
+    int c_id = active_client->clientGroup()->indexOfClient( active_client );
+    int size = active_client->clientGroup()->clients().count();
+    if( c_id < size - 1 )
+        active_client->clientGroup()->setVisible( c_id + 1 );
+    else
+        active_client->clientGroup()->setVisible( 0 );
+    }
+
+void Workspace::slotSwitchToTabLeft()
+    {
+    if( !active_client || !active_client->clientGroup() )
+        return;
+    int c_id = active_client->clientGroup()->indexOfClient( active_client );
+    int size = active_client->clientGroup()->clients().count();
+    if( c_id > 0 )
+        active_client->clientGroup()->setVisible( c_id - 1 );
+    else
+        active_client->clientGroup()->setVisible( size - 1 );
+    }
+
+void Workspace::slotRemoveFromGroup()
+    {
+    if( !active_client || !active_client->clientGroup() )
+        return;
+    active_client->clientGroup()->remove( active_client );
+    }
 
 /*!
   Kill Window feature, similar to xkill
@@ -1165,6 +1455,104 @@ void Workspace::slotSendToDesktop( QAction *action )
 
     }
 
+/*!
+  Switches to the nearest window in given direction
+ */
+void Workspace::switchWindow( Direction direction )
+    {
+    if( !active_client )
+        return;
+    Client *c = active_client;
+    Client *switchTo = 0;
+    int bestScore = 0;
+    int d = c->desktop();
+    // Centre of the active window
+    QPoint curPos( c->pos().x() + c->geometry().width() / 2,
+                   c->pos().y() + c->geometry().height() / 2 );
+
+    QList<Client *> clist = stackingOrder();
+    for( QList<Client *>::Iterator i = clist.begin(); i != clist.end(); ++i )
+        {
+        if( (*i)->wantsTabFocus() && *i != c &&
+            (*i)->desktop() == d && ! (*i)->isMinimized() )
+            {
+            // Centre of the other window
+            QPoint other( (*i)->pos().x() + (*i)->geometry().width() / 2,
+                          (*i)->pos().y() + (*i)->geometry().height() / 2 );
+
+            int distance;
+            int offset;
+            switch( direction )
+                {
+                case DirectionNorth:
+                    distance = curPos.y() - other.y();
+                    offset = qAbs(other.x() - curPos.x());
+                    break;
+                case DirectionEast:
+                    distance = other.x() - curPos.x();
+                    offset = qAbs(other.y() - curPos.y());
+                    break;
+                case DirectionSouth:
+                    distance = other.y() - curPos.y();
+                    offset = qAbs(other.x() - curPos.x());
+                    break;
+                case DirectionWest:
+                    distance = curPos.x() - other.x();
+                    offset = qAbs(other.y() - curPos.y());
+                    break;
+                default:
+                    distance = -1;
+                    offset = -1;
+                }
+
+            if( distance > 0 )
+                {
+                // Inverse score
+                int score = distance + offset + ( (offset * offset) / distance );
+                if( score < bestScore || !switchTo )
+                    {
+                    switchTo = *i;
+                    bestScore = score;
+                    }
+                }
+            }
+        }
+    if( switchTo )
+        activateClient( switchTo );
+    }
+
+/*!
+  Switches to upper window
+ */
+void Workspace::slotSwitchWindowUp()
+    {
+    switchWindow( DirectionNorth );
+    }
+
+/*!
+  Switches to lower window
+ */
+void Workspace::slotSwitchWindowDown()
+    {
+    switchWindow( DirectionSouth );
+    }
+
+/*!
+  Switches to window on the right
+ */
+void Workspace::slotSwitchWindowRight()
+    {
+    switchWindow( DirectionEast );
+    }
+
+/*!
+  Switches to window on the left
+ */
+void Workspace::slotSwitchWindowLeft()
+    {
+    switchWindow( DirectionWest );
+    }
+
 /*!
   Shows the window operations popup menu for the activeClient()
  */
@@ -1216,8 +1604,9 @@ void Workspace::showWindowMenu( const QRect &pos, Client* cl )
  */
 void Workspace::slotWindowClose()
     {
-    if ( tab_box->isVisible())
-        return;
+    // TODO: why?
+//     if ( tab_box->isVisible())
+//         return;
     Client* c = active_popup_client ? active_popup_client : active_client;
     performWindowOperation( c, Options::CloseOp );
     }
