@@ -27,6 +27,9 @@
 
 #include <KDE/KCalendarSystem>
 
+class KConfigDialog;
+class KConfigGroup;
+
 namespace Plasma
 {
 
@@ -48,21 +51,35 @@ public:
     Q_DECLARE_FLAGS(CellTypes, CellType)
 
     explicit CalendarTable(QGraphicsWidget *parent = 0);
-    CalendarTable(const QDate &, QGraphicsWidget *parent = 0);
+    explicit CalendarTable(const QDate &, QGraphicsWidget *parent = 0);
     ~CalendarTable();
 
-    bool setCalendar(KCalendarSystem *calendar = 0);
+    bool setCalendar(const QString &newCalendarType = "locale");
+    bool setCalendar(const KCalendarSystem *calendar);
     const KCalendarSystem *calendar () const;
 
     bool setDate(const QDate &date);
     const QDate& date() const;
 
     void setDataEngine(Plasma::DataEngine *dataEngine);
-    void setRegion(const QString &region);
+    const Plasma::DataEngine *dataEngine() const;
+
+    bool setDisplayHolidays(bool showHolidays);
+    bool displayHolidays();
+
+    bool setHolidaysRegion(const QString &region);
+    QString holidaysRegion() const;
+
     void clearDateProperties();
-    void setDateProperty(QDate date, const QString &reason); //HACK
+    void setDateProperty(QDate date, const QString &reason);
     QString dateProperty(QDate date) const;
 
+    void applyConfiguration(KConfigGroup cg);
+    void writeConfiguration(KConfigGroup cg);
+    void createConfigurationInterface(KConfigDialog *parent);
+    void applyConfigurationInterface();
+    void configAccepted(KConfigGroup cg);
+
 Q_SIGNALS:
     void dateChanged(const QDate &newDate, const QDate &oldDate);
     void dateChanged(const QDate &newDate);
@@ -92,4 +109,5 @@ private:
 }
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(Plasma::CalendarTable::CellTypes)
+
 #endif
