@@ -250,6 +250,14 @@ QStringList NMNetworkManager::activeConnections() const
     return activeConnections;
 }
 
+Solid::Control::NetworkInterface::Types NMNetworkManager::supportedInterfaceTypes() const
+{
+    return (Solid::Control::NetworkInterface::Types) (
+           Solid::Control::NetworkInterface::Ieee80211 |
+           Solid::Control::NetworkInterface::Ieee8023
+           );
+}
+
 void NMNetworkManager::activateConnection(const QString & interfaceUni, const QString & connectionUni, const QVariantMap & connectionParameters)
 {
     kDebug(1441) << interfaceUni << connectionUni << connectionParameters;
